/*
	core.js
	Copyright © 2005 - 2013  WOT Services Oy <info@mywot.com>

	This file is part of WOT.

	WOT is free software: you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	WOT is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
	or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
	License for more details.

	You should have received a copy of the GNU General Public License
	along with WOT. If not, see <http://www.gnu.org/licenses/>.
*/

"use strict";

function wot_listener(browser)
{
	this.browser = browser;
}

wot_listener.prototype =
{
	browser: null,

	aborted: 0x804b0002, /* NS_BINDING_ABORTED */

	loading: Components.interfaces.nsIWebProgressListener.STATE_START |
			 Components.interfaces.nsIWebProgressListener.STATE_REDIRECTING |
			 Components.interfaces.nsIWebProgressListener.STATE_TRANSFERRING |
			 Components.interfaces.nsIWebProgressListener.STATE_NEGOTIATING,

	finish_loading: Components.interfaces.nsIWebProgressListener.STATE_STOP ,

	same_document : Components.interfaces.nsIWebProgressListener.LOCATION_CHANGE_SAME_DOCUMENT,

	isdocument: Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT,

	NavigationData: null,
	RealTimeAPI: null,

	init: function() {
		this.import_modules();
	},

	import_modules: function() {
		this.NavigationData = wot_core.import_module("resource://wot-chrome-modules/NavigationData.js");
		this.RealTimeAPI = wot_core.import_module("resource://wot-chrome-modules/RealTimeAPI.js");
	},

	abort: function(request)
	{
		request.cancel(this.aborted);
	},

	QueryInterface: function(iid)
	{
		if (!iid.equals(Components.interfaces.nsISupports) &&
			!iid.equals(Components.interfaces.nsISupportsWeakReference) &&
			!iid.equals(Components.interfaces.nsIWebProgressListener)) {
			throw Components.results.NS_ERROR_NO_INTERFACE;
		}

		return this;
	},

	onLocationChange: function(browser, progress, request, location)
	{
		// browser - type="content-primary"
		try {
			if (progress.DOMWindow != browser.contentWindow) {
				return;
			}

			var tabUrl = location.spec;
			if (location && wot_core.is_rtp_on()) {
				wot_core.block(this, request, tabUrl, browser);
			}
			var navigationData = null;

			if (tabUrl && wot_stats.isWebURL(tabUrl)) {
				var ref = browser.contentDocument.referrer;
				if (request && request.referrer && typeof(request.referrer) != undefined) {
					ref = request.referrer.asciiSpec;
				}

				wot_stats.initLastPrev();
				var last_prev = wot_stats.getLastPrev();
				wot_stats.setLastPrev(tabUrl);

				navigationData = new this.NavigationData(ref, tabUrl, last_prev);

				var options = { 
					base64Encode: true
					// encodeURIComponent: true
				};

				var rtQuery = this.RealTimeAPI.QueryBuilder.getQuery(
					navigationData, options
				);

				navigationData.setRtQuery(rtQuery);
				// console.log(navigationData);
			}

			wot_core.update(browser, navigationData);
		} catch(e) { 
			console.log(e);
		}
	},

	onProgressChange: function(progress, request, curSelfProgress,
		maxSelfProgress, curTotalProgress, maxTotalProgress)
	{
	},

	onStateChange: function(browser, progress, request, flags, status)
	{
		try {
			if (progress.DOMWindow != browser.contentWindow) {
				return;
			}
			if (flags & this.loading && flags & this.isdocument && request && wot_core.is_rtp_on()) {
				wot_core.block(this, request, request.name, browser);
			}
		} catch(e) { }
	},

	onStatusChange: function(browser, webProgress, request, status, message)
	{
	},

	onSecurityChange: function(browser, progress, request, state)
	{
	},

	// onRefreshAttempted: function(browser, webProgress, refrushURI, millis, sameUri)
	// {
	// },

	onLinkIconAvailable: function(browser)
	{
	}
};

var wot_core = new function() {
	var { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;

	var DebugConfig;
	var NavigationData;
	var RealTimeAPI;

	this.blockedstreams = {};
	this.hostname = null;
	this.pending = {};
	this.purged = Date.now();
	this.loaded = false;
	this.is_beingUninstalled = false;

	this.import_modules = function() {
		DebugConfig = this.import_module("resource://wot-chrome-modules/DebugConfig.js", false);
		NavigationData = this.import_module("resource://wot-chrome-modules/NavigationData.js");
		RealTimeAPI = this.import_module("resource://wot-chrome-modules/RealTimeAPI.js");
	};

	this.import_module = function(resource_path, is_expose_globally) {
		var module_temp = {};

		// for debugging purposes
		if (is_expose_globally !== false) {
			is_expose_globally = DebugConfig.IS_DEBUG_ON;
		}

		Cu.import(resource_path, module_temp);
		var slash_last_index = resource_path.lastIndexOf("/");
		var module_name = resource_path.substr(slash_last_index + 1);
		module_name = module_name.split(".js")[0];

		var module = module_temp[module_name];

		if (is_expose_globally) {
			window[module_name] = module;
		}

		return module;
	};

	this.init = function() {
		try {
			this.import_modules();

			window.addEventListener("load", function on_load (e) {
					window.removeEventListener("load", on_load, true);
					wot_core.load();
				}, false);

			window.addEventListener("unload", function(e) {
					wot_core.unload();
				}, false);

			window.addEventListener("sizemodechange", function (e) {
				wot_warning.blur();
			}, false);

			window.addEventListener("deactivate", function (e) {
				wot_warning.blur(true);
			}, false);

			window.addEventListener("activate", function (e) {
				wot_warning.blur();
			}, false);

			this.browser = document.getElementById("appcontent");

			if (this.browser) {
				this.browser.addEventListener("DOMContentLoaded", wot_core.domcontentloaded, false);
				this.browser.addEventListener("pageshow", wot_core.pageshow, false);    // the fix for the issue with Back/Forward buttons
				this.browser.addEventListener("click", wot_core.click, false);
			}
		} catch (e) {
			console.log("wot_core.init: failed with " + e + "\n");
		}
	};

	this.load = function() {
		try {
			for (var i in wot_modules) {
				if (typeof(wot_modules[i].obj.load) == "function") {
					wot_modules[i].obj.load();
				}
			}

			wot_storage.init(function() {   // continue initialization only after the storage is loaded
				// schedule delayed load of modules ()
				window.setTimeout(function() {
					for (var i in wot_modules) {
						if (typeof(wot_modules[i].obj.load_delayed) == "function") {
							wot_modules[i].obj.load_delayed();
						}
					}

					wot_prefs.setupdateui();
					wot_api_register.send();
					wot_api_register.send("rt_api", "rt_witness_id", "rt_witness_key");
					wot_api_active.send("rt_api");
					wot_my_session.update(true);

					wot_core.loaded = true;
					wot_ui.init();
					wot_core.update();
				}, 250);

				try {
					Components.utils.import("resource://gre/modules/AddonManager.jsm");
					AddonManager.addAddonListener(wot_core.install_listener);
				} catch (e) {
					dump("wot_core.load() setting up uninstall listener failed with " + e + "\n");
				}

				var browser = getBrowser();
				wot_core.listener = new wot_listener(browser);
				wot_core.listener.init();
				browser.addTabsProgressListener(wot_core.listener);

				if (browser.tabContainer) {
					browser.tabContainer.addEventListener("TabSelect",
						wot_core.tabselect, false);
				}

				setTimeout(() => {
					RealTimeAPI.init();	// need to wait for all the core modules to init
				}, 500);
			});

		} catch (e) {
			console.log("wot_core.load: failed with " + e + "\n");
		}
	};

	this.install_listener = {
		onUninstalling: function(addon) {
			if (addon.id == WOT_GUID) {
				wot_core.is_beingUninstalled = true;
			}
		},
		onOperationCancelled: function(addon) {
			if (addon.id == WOT_GUID) {
				wot_core.is_beingUninstalled = (addon.pendingOperations & AddonManager.PENDING_UNINSTALL) != 0;
			}
		}
	};

	this.unload = function() {
		try {
			if (wot_core.is_beingUninstalled) {
				wot_core.clean_firstrun_guide();
			}			

			var browser = getBrowser();

			if (this.listener) {
				browser.removeTabsProgressListener(this.listener);
				this.listener = null;
			}

			if (browser.tabContainer) {
				browser.tabContainer.removeEventListener("TabSelect",
					wot_core.tabselect, false);
			}

			if (this.browser) {
				this.browser.removeEventListener("DOMContentLoaded",
						wot_core.domcontentloaded, false);
				this.browser.removeEventListener("click",
						wot_core.click, false);
				this.browser = null;
			}

			// do pre-cleaning on uninstall (before modules are unloaded)
			if (wot_core.is_beingUninstalled) {
				wot_core.clean_search_rules();
			}

			for (var i = wot_modules.length - 1; i >= 0; i--) {
				if (typeof(wot_modules[i].obj.unload) == "function") {
					wot_modules[i].obj.unload();
				}
			}

			// do past-cleaning on uninstall
			if (wot_core.is_beingUninstalled) {
				wot_core.clean_profile_dir();
			}

		} catch (e) {
			dump("wot_core.unload: failed with " + e + "\n");
		}
	};

	this.domcontentloaded = function(event, retry) {
		if (!wot_core.loaded && !retry) {
			window.setTimeout(function() {
					wot_core.domcontentloaded(event, true);
				}, 500);
		}

		for (var i in wot_modules) {
			if (typeof(wot_modules[i].obj.domcontentloaded) == "function") {
				wot_modules[i].obj.domcontentloaded(event);
			}
		}
	};

	this.pageshow = function(event, retry) {
		if (!wot_core.loaded && !retry) {
			window.setTimeout(function() {
					wot_core.pageshow(event, true);
				}, 500);
		}

		for (var i in wot_modules) {
			if (typeof(wot_modules[i].obj.pageshow) == "function") {
				wot_modules[i].obj.pageshow(event);
			}
		}
	};

	this.click = function(event) {
		for (var i in wot_modules) {
			if (typeof(wot_modules[i].obj.click) == "function") {
				wot_modules[i].obj.click(event);
			}
		}
	};

	this.tabselect = function(event) {
		try {
			var browser = getBrowser().selectedTab;
			var tabUrl = event.target.linkedBrowser.currentURI.spec;
			wot_stats.focus(tabUrl);
			wot_core.update(browser);			
		} catch (e) {
			dump("wot_core.tabselect: failed with " + e + "\n");
		}
	};

	this.showloading = function(pl, request, url, hostname, browser) {
		try {
			var stream = null;
			if (request) {
				if (request.QueryInterface) {
					var channel = request.QueryInterface(
									Components.interfaces.nsIHttpChannel);

					if (channel && channel.requestMethod == "POST") {
						var upload = request.QueryInterface(
										Components.interfaces.nsIUploadChannel);

						if (upload) {
							stream = upload.uploadStream;
						}
					}
				}

				pl.abort(request);
			}

			if (wot_browser.isoffline()) {
				return;
			}

			this.blockedstreams[url] = stream;

			browser.loadURIWithFlags(WOT_BLOCK_LOADING + "#" +
				encodeURIComponent(btoa(url)),
				Components.interfaces.nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY,
				null, null);

				wot_api_query.send({ 
					hostname: wot_core.hostname 
				});
		} catch (e) {
			dump("wot_core.showloading: failed with " + e + "\n");
		}
	};

	this.showblocked = function(pl, request, url, hostname, browser) {
		try {
			if (request) {
				pl.abort(request);
			}

			var blocked = "target=" + encodeURIComponent(hostname);

			for (var i = 0, a = 0; i < WOT_COMPONENTS.length; ++i) {
                a = WOT_COMPONENTS[i];
				if (!wot_prefs["show_application_" + a]) {
					continue;
				}

				var param = "";
				var reason = wot_warning.getwarningtype(hostname, a, true);

				if (reason == WOT_REASON_TESTIMONY) {
					param += "y";
				} else if (reason == WOT_REASON_RATING) {
					param += "r";
				}

				var r = wot_cache.get(hostname, "reputation_" + a),
                    x = wot_cache.get(hostname, "excluded_" + a);

                r = x ? -2 : r; // if excluded, then set level to -2

				param += wot_util.get_level(WOT_REPUTATIONLEVELS, r).level;

				if (wot_prefs.accessible) {
					param += "a";
				}

				blocked += "&" + a + "=" + param;
			}

			blocked = "?" + encodeURIComponent(btoa(blocked)) + "#" +
						encodeURIComponent(btoa(url));

			browser.loadURI(WOT_BLOCK_BLOCKED + blocked);
		} catch (e) {
			dump("wot_core.showblocked: failed with " + e + "\n");
		}
	};

	this.block = function(pl, request, url, browser) {
		try {
			if (!wot_util.isenabled() || !pl || !pl.browser || !url) {
				return false;
			}

			if (!wot_warning.isblocking()) {
				return false;
			}

			var hostname = wot_url.gethostname(url);

			if (!hostname || wot_url.isprivate(hostname) ||
					wot_url.isexcluded(hostname)) {
				return false;
			}

			if (wot_cache.isok(hostname)) {
				if (wot_warning.isdangerous(hostname, false) ==
						WOT_WARNING_BLOCK) {
					this.showblocked(pl, request, url, hostname, browser);
				}

				if (this.blockedstreams[url]) {
					delete this.blockedstreams[url];
				}
			} else {
				this.showloading(pl, request, url, hostname, browser);
			}

			return true;
		} catch (e) {
			dump("wot_core.block: failed with " + e + "\n");
		}
	};

	this.updateloading = function() {
		try {
			if (!wot_warning.isblocking()) {
				return;
			}

			var browser = getBrowser();
			var num = browser.mPanelContainer.childNodes.length;

			for (var i = 0; i < num; i++) {
				var tab = browser.getBrowserAtIndex(i);

				if (!tab || !tab.currentURI ||
						tab.currentURI.spec.indexOf(WOT_BLOCK_LOADING) != 0) {
					continue;
				}

				var url = this.isredirect(tab.currentURI.spec);

				if (!url) {
					continue;
				}

				var hostname = wot_url.gethostname(url);

				if (!hostname || !wot_cache.iscached(hostname)) {
					continue;
				}

				var age = Date.now() - wot_cache.get(hostname, "time");

				if (wot_cache.get(hostname, "status") == WOT_QUERY_ERROR &&
						age < WOT_INTERVAL_BLOCK_ERROR) {
					continue;
				}

				var postdata = null;

				if (this.blockedstreams[url]) {
					postdata = this.blockedstreams[url];
					delete this.blockedstreams[url];
				}

				/* Try again */
				tab.loadURIWithFlags(url,
					Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE,
					null, null, postdata);
			}
		} catch (e) {
			console.log("wot_core.updateloading: failed with " + e + "\n");
		}
	};

	this.is_internal = function(url) {
		return (url.indexOf(WOT_BLOCK_LOADING) >= 0 || url.indexOf(WOT_BLOCK_BLOCKED) >= 0);
	};

	this.isredirect = function(url) {
		// on the Blocked page we extract encoded hostname from parameter, and use it as a target
		try {
			if (!url) return null;

			if(!this.is_internal(url)) return null;

			var m = /#(.+)$/.exec(url);

			if (m && m[1]) {
				return atob(decodeURIComponent(m[1]));
			}
		} catch (e) {
			dump("wot_core.isredirect: failed with " + e + "\n");
		}
		return null;
	};

	this.purgecache = function() {
		try {
			var interval = WOT_INTERVAL_CACHE_REFRESH;

			/* Purging cache entries while blocking is enabled causes the
				page to be reloaded while ratings are being loaded, so we'll
				purge the cache less often to not annoy the user... */

			if (wot_warning.isblocking()) {
				interval = WOT_INTERVAL_CACHE_REFRESH_BLOCK;
			}

			var now = Date.now();

			if ((now - wot_core.purged) < interval) {
				return;
			}

			wot_core.purged = now;
			var cache = wot_cache.get_enumerator();

			while (cache.hasMoreElements()) {
				var name = wot_cache.get_name_from_element(cache.getNext());

				if (!name) {
					continue;
				}

				if (wot_cache.get(name, "inprogress")) {
					continue;
				}

				var age = now - wot_cache.get(name, "time");

				if (age > interval) {
					wot_cache.destroy(name);
				}
			}
		} catch (e) {
			dump("wot_core.purgecache: failed with " + e + "\n");
		}
	};

	this.is_rtp_on = function() {
		return wot_prefs.getBool("settingsui_dynamicprotect") || false;
	};	

	this.update = function(browser, navigationData) {
		try {
			if (this.is_beingUninstalled) { return; }
			browser = browser || null;
			navigationData = navigationData || null;
			wot_core.hostname = null;

			// register api is not ready / wot_core has not been loaded yet
			if (!wot_api_register.ready.api || !wot_core.loaded) {
				wot_status.set("notready",
					wot_util.getstring("messages_notready"));
				return;
			}

			if (!this.is_rtp_on()) {
				wot_status.set("rtp-off", wot_util.getstring("messages_rtpoff"));
			}

			// no witness id/key found
			if (!wot_prefs.witness_id || !wot_prefs.witness_key ||
					wot_prefs.witness_id.length  != WOT_LENGTH_WITNESS_ID ||
					wot_prefs.witness_key.length != WOT_LENGTH_WITNESS_KEY) {
				wot_api_register.ready.api = false;
				wot_status.set("error",
					wot_util.getstring("description_restart"));
				return;
			}

			wot_core.hostname = wot_browser.gethostname();

			/* Update session */
			if (browser && wot_core.hostname && WOT_MY_TRIGGER.test(wot_core.hostname)) {
				wot_my_session.update(false);
				wot_my_session.reload();

				var url = wot_browser.geturl();
				var match = url.match(WOT_PREF_FORWARD);

				if (match) {
					var section = match[WOT_PREF_FORWARD_TAB_MATCH];
                    var base = (match[WOT_PREF_FORWARD_TAB_BASE] + "/settings/") || WOT_PREF_PATH;

                        browser.loadURIWithFlags(wot_url.getprefurl(section, base),
						Components.interfaces.nsIWebNavigation
							.LOAD_FLAGS_BYPASS_HISTORY, null, null);
				}
			}

			/* Check for updates (force update if no categories are loaded yet) */
            var forced_update = wot_util.isEmpty(wot_categories.categories) ||
                !wot_categories.grouping ||
                wot_categories.grouping.length == 0;

			wot_api_update.send(forced_update);			

			if (this.is_rtp_on()) {
				this.update_with_rtp_on(browser, navigationData);
				return;
			}

			this.update_with_rtp_off();

		} catch(ex) { 
			console.log(ex);
		}
	};

	this.update_with_rtp_on = function(browser, navigationData) {
		try {
			// show popup panel
			wot_ui.showpanel();
			wot_ui.hide_rtp_view();
			wot_ui.remove_rtp_off_click_event();

			// not enabled
			if (!wot_util.isenabled()) {
				wot_status.set("disabled",
					wot_util.getstring("messages_disabled"));
				return;
			}

			/* Store any pending testimonies (from this window) */
			for (var i in wot_core.pending)  {
				if (wot_pending.store(i)) {
					wot_cache.set(i, "pending", false);
					delete wot_core.pending[i];
				}
			}

			/* Update any tabs waiting for reputations */
			wot_core.updateloading();

			/* Recover the original hostname */
			var redirected = wot_core.isredirect(wot_browser.geturl());
			if (redirected) {
				wot_core.hostname = wot_url.gethostname(redirected);
			}

			/* Purge expired cache entries */
			wot_core.purgecache();

			if (wot_browser.isoffline()) {
				/* Browser offline */
				wot_status.set("offline",
					wot_util.getstring("message_offline"));
				/* Retry after a timeout */
				window.setTimeout(wot_core.update, WOT_INTERVAL_UPDATE_OFFLINE);
				return;
			}

			/* Submit any pending testimonies, comments and comments' removals */
			wot_pending.submit();
            wot_api_comments.processpending();

			wot_wg.update_tags();   // update user tags and popular tags

			if (!wot_core.hostname || wot_url.isprivate(wot_core.hostname) ||
					wot_url.isexcluded(wot_core.hostname)) {
				/* Invalid or excluded hostname */
				if (wot_cache.iscached(wot_core.hostname) &&
						!wot_cache.get(wot_core.hostname, "pending")) {
					wot_cache.destroy(wot_core.hostname);
				}
				wot_status.set("nohost",
					wot_util.getstring("messages_notavailable"));

				wot_status.update();
				return;
			}

			// hostname is not cached
			if (!wot_cache.iscached(wot_core.hostname)) {
				/* No previous record of the hostname, start a new query */
				wot_status.set("inprogress",
					wot_util.getstring("messages_loading"));
				wot_api_query.send({ 
					navigationData,
					hostname: wot_core.hostname	// will use the hostname incase 'navigationData' is null
				});

				return;
			} else {
				wot_api_query.send({ 
					navigationData,
					is_rt_query: true
				});
			}			

			var age = Date.now() - wot_cache.get(wot_core.hostname, "time");

			if (wot_cache.get(wot_core.hostname, "inprogress")) {
				if (age > WOT_TIMEOUT_QUERY) {
					/* Done waiting, clear the flag  */
					wot_cache.set(wot_core.hostname, "inprogress", false);
				} else {
					/* Query already in progress, keep waiting */
					wot_status.set("inprogress",
						wot_util.getstring("messages_loading"));
					return;
				}
			}

			var status = wot_cache.get(wot_core.hostname, "status");

			if (status == WOT_QUERY_OK) {
				if (age > WOT_INTERVAL_CACHE_REFRESH) {
					wot_status.set("inprogress",
						wot_util.getstring("messages_loading"));
					wot_api_query.send({ 
						hostname: wot_core.hostname 
					});
				} else {
					wot_status.update();	// add browser ?
				}
				return;
			} else if (!wot_cache.get(wot_core.hostname, "pending")) {
				if (status == WOT_QUERY_RETRY || status == WOT_QUERY_LINK) {
					/* Retry immediately */
					wot_status.set("inprogress",
						wot_util.getstring("messages_loading"));
					wot_api_query.send({ 
						hostname: wot_core.hostname 
					});
					return;
				} else if (age > WOT_INTERVAL_CACHE_REFRESH_ERROR) {
					wot_status.set("inprogress",
						wot_util.getstring("messages_loading"));
					wot_api_query.send({ 
						hostname: wot_core.hostname 
					});
					return;
				}
			}
		} catch (e) {
			console.log("wot_core.update_with_rtp_on: failed with " + e + "\n");
		}

		try {
			/* For some reason, we failed to get anything meaningful to display */
			wot_status.set("error",
				wot_util.getstring("messages_failed"));
		} catch (e) {
			console.log("wot_core.update_with_rtp_on: failed with " + e + "\n");
		}
	};

	this.update_with_rtp_off = function() {
		if (!wot_prefs.rtp_has_user_selected) {
			wot_ui.show_rtp_view();
			if (!wot_prefs.rtp_automatically_shown && !wot_ui.is_popup_panel_open()) {
				wot_ui.actively_show_rtp_popup();	// will only be shown if never been shown before
				wot_prefs.setBool("rtp_automatically_shown", true);
			}

			return;
		}

		wot_ui.hidepanel();
		wot_ui.add_rtp_off_click_event();	// open scorecard instead of regular popup
	};

	this.clean_search_rules = function () {
		// removes search rules from preferences
		wot_prefs.deleteBranch(WOT_SEARCH);
		wot_prefs.clear("update_checked");
	};

	this.clean_firstrun_guide = function() {
		wot_prefs.clear("rtp_automatically_shown");
		wot_prefs.clear("rtp_has_user_selected");
		wot_prefs.clear("settingsui_dynamicprotect");
		wot_prefs.clear("firstrun_time");
		wot_prefs.clear("firstrun_guide");
		wot_prefs.clear("stats_sess");
		wot_prefs.clear("ratingwindow_shown");
	};

	this.clean_profile_dir = function() {
		// removes only WOT dir in the profile folder
		try {
			var nsiDir = FileUtils.getDir("ProfD", [wot_file.wot_dir], false);
			if (nsiDir && nsiDir.exists()) {
				nsiDir.remove(true);    // 'true' for recursive removal
			}
		} catch (e) {
			dump("wot_core.clean_profile_dir() failed with " + e + "\n");
		}
	};
};

wot_core.init();