try {
var EXPORTED_SYMBOLS = ["RealTimeAPI"];
var { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;

Cu.import("resource://wot-chrome-modules/log.js");
Cu.import("resource://wot-chrome-modules/Utils.js");
Cu.import("resource://wot-chrome-modules/CoreModuleGetter.js");

var DEBUG_ON = true;
if (!DEBUG_ON) { log = () => {}; warn = log; }

var wot_prefs;
var wot_crypto;
var wot_url;
// var wot_pi_filter;

const RT_WITNESS_ID = "rt_witness_id";
const RT_WITNESS_KEY = "rt_witness_key";
const WG_PARAM = "wg=1";
const RT_PREF = "settingsui_dynamicprotect";

// taken from "config.js"
const WOT_LENGTH_WITNESS_ID = 40;
const WOT_LENGTH_WITNESS_KEY  = 40;
const WOT_SERVICE_RT_API_SG = "241";
const WOT_SERVICE_RT_API_VERSION = "2.0.6";

var RealTimeAPI = new function() {
    this.queryBuilder;

    this.init = function() {
        var recentWin = Utils.getMostRecentWindow();

        wot_prefs = CoreModuleGetter.getModule("wot_prefs", this.recentWin);
        wot_crypto = CoreModuleGetter.getModule("wot_crypto", this.recentWin);
        wot_url = CoreModuleGetter.getModule("wot_url", this.recentWin);
        wot_pi_filter = CoreModuleGetter.getModule("wot_pi_filter", this.recentWin);

        this.queryBuilder = new QueryBuilder();
    };

    Object.defineProperty(this, "QueryBuilder", { 
        get: function() { return this.queryBuilder ? this.queryBuilder : null; }
    });


    this.isRTWitnessIdExist = function() {
        return wot_prefs.hasOwnProperty(RT_WITNESS_ID);
    };

    this.isRTWitnessKeyExist = function() {
        return wot_prefs.hasOwnProperty(RT_WITNESS_KEY);
    };    

    this.getRTWitnessId = function() {
        return this.isRTWitnessIdExist() ? wot_prefs[RT_WITNESS_ID] : null;
    };

    this.getRTWitnessKey = function() {
        return this.isRTWitnessKeyExist() ? wot_prefs[RT_WITNESS_KEY] : null;
    };

    this.isRTWitnessIdValid = function() {
        var witnessId = this.getRTWitnessId();

        return witnessId !== null && witnessId.length === WOT_LENGTH_WITNESS_ID;
    };

    this.isRTWitnessKeyValid = function() {
        var witnessKey = this.getRTWitnessKey();

        return witnessKey !== null && witnessKey.length === WOT_LENGTH_WITNESS_KEY;
    };
};

RealTimeAPI.RT_WITNESS_ID = RT_WITNESS_ID;
RealTimeAPI.RT_WITNESS_KEY = RT_WITNESS_KEY;
RealTimeAPI.WG_PARAM = WG_PARAM;
RealTimeAPI.WG_PARAM = WG_PARAM;

function QueryBuilder() {
    // params
    const ID = "id";
    const NONCE = "nonce";
    const LANG = "lang";
    const VERSION = "version";
    const TARGET = "target";

    const TARGET_PARAMS = {
        SUBTARGET: "subtrgt",
        SUBREF: "subref",
        SUBLAST: "sublast",
        ID: "id",
        SG: "sg"
    };

    this.targetId;

    this.init = function() {
        this.targetId = new TargetId();     // retrieves the right target id for the user
    };

    this.getAPILangAndVersionParams = wot_url.getapiparams;   // example - "&lang=en-US&version=firefox-20151208"

    // navigationData = { DocReferrerURL, DocURL, LastURLNavigation }
    this.getQuery = function(navigationData, options) {
        var queryParams = {};

        // add error codes
        var id = RealTimeAPI.getRTWitnessId();      // id
        log(id);
        if (!id) { return ""; }

        var nonce = this.getNonce(id);              // nonce
        log(nonce);
        if (!nonce) { return ""; }

        var version = this.getAPIParam(VERSION);    // version
        log(version);
        if (!version) { return ""; }

        var lang = this.getAPIParam(LANG);          // lang
        log(lang);
        if (!lang) { return ""; }

        var target = this.getTarget(navigationData, nonce, true);   // true = encrypts the data
        log(target);
        if (!target) { return ""; }

        log(id);
        log(nonce);
        log(version);
        log(lang);
        log(target);
        log(this.encrypt(target, nonce));

        queryParams[ID] = id;
        queryParams[NONCE] = nonce;
        queryParams[VERSION] = version;
        queryParams[LANG] = lang;
        queryParams[TARGET] = target;

        var query = this.buildQueryString(queryParams);
        log(query);

        if (options) {
            if (options.base64Encode) {
                query = Utils.base64Encode(query);
            }

            if (options.encodeURIComponent) {
                query = Utils.encodeURIComponent(query);
            }
        }

        return query;
    };

    this.getTarget = function(navigationData, nonce, isEncrypt) {
        // subtrgt, subref, sublast
        var queryTargetParams = this.convertNavigationDataToQueryTargetParams(navigationData);

        // adds the target "id"
        var id = this.targetId.get();
        log(id);
        queryTargetParams[TARGET_PARAMS.ID] = id;
        queryTargetParams[TARGET_PARAMS.SG] = WOT_SERVICE_RT_API_SG;

        if (!this.validateQueryTargetParams(queryTargetParams)) {
            return "";
        }

        var target = this.buildQueryString(queryTargetParams);

        if (isEncrypt) {
            var encryptedTarget = this.encrypt(target, nonce);

            return encryptedTarget;
        }

        return target;
    };

    this.buildQueryString = function(params, isOmitEmptyValues) {
        var components = [];

        for (var name in params) {
            var value = params[name];
            if (isOmitEmptyValues && value === "") { 
                continue;
            }
            components.push(name + "=" + encodeURIComponent(value));
        }

        return components.join("&");
    };

    this.encrypt = function(data, nonce) {
        var witnessKey = RealTimeAPI.getRTWitnessKey();
        if (!witnessKey) {
            log(new Error("RealTimeAPI.QueryBuilder.encrypt: 'rt_witness_key' is undefined"))
            return null;
        }

        return wot_crypto.encrypt(data, nonce, witnessKey);
    };

    this.getNonce = function(witnessId) {
        return wot_crypto.nonce(witnessId);
    };

    this.getId = function() {
        var witnessId = RealTimeAPI.getRTWitnessId();
        if (!witnessId) {
           log(new Error("RealTimeAPI.QueryBuilder.getNonce: 'rt_witness_id' is undefined"));
           return null;
        }

        return witnessId;
    };

    // 'lang' or 'version'
    this.getAPIParam = function(paramName) {
        try {
            var APIParamsString = this.getAPILangAndVersionParams();  // lang, version

            if (!this.validateAPIParamsString(APIParamsString)) {
                log(new Error("RealTimeAPI.QueryBuilder.validateAPIParamsString: failed to get '" + paramName + "' param"));
                return "";
            }
            var version = APIParamsString.split("&")[2].split("=")[1];
            var lang = APIParamsString.split("&")[1].split("=")[1];

            if (paramName === VERSION) { 
                return this.appendRtAPIVersion(version);
            }
            if (paramName === LANG) {
                return lang;
            }
            return "";
        } catch(ex) {
            log(ex);
            return "";
        }
    };

    this.appendRtAPIVersion = function(version) {
        return version + "-" + WOT_SERVICE_RT_API_VERSION;
    };

    // a simple verification - check if "&" and "=" exist
    this.validateAPIParamsString = function(APIParamsString) {
        if (APIParamsString.indexOf("&") === -1 
            || APIParamsString.indexOf("=") === -1) {
            return false;
        }

        return true;
    };

    this.convertNavigationDataToQueryTargetParams = function(navigationData) {
        if (!navigationData) {
            log(new Error("RealTimeAPI.QueryBuilder.convertNavigationDataToQueryParams: navigationData is missing"));
            return null;
        }
        var queryTargetParams = {};
        queryTargetParams[TARGET_PARAMS.SUBTARGET] = wot_pi_filter.filter_all(navigationData.DocURL);
        queryTargetParams[TARGET_PARAMS.SUBLAST] = wot_pi_filter.filter_all(navigationData.LastURLNavigation) || "";
        queryTargetParams[TARGET_PARAMS.SUBREF] = wot_pi_filter.filter_all(navigationData.DocReferrerURL) || "";

        // queryTargetParams[TARGET_PARAMS.SUBTARGET] = navigationData.DocURL;
        // queryTargetParams[TARGET_PARAMS.SUBLAST] = navigationData.LastURLNavigation;
        // queryTargetParams[TARGET_PARAMS.SUBREF] = navigationData.DocReferrerURL;

        return queryTargetParams;
    };

    this.validateQueryTargetParams = function(queryTargetParams) {
        if (queryTargetParams[TARGET_PARAMS.SUBTARGET] === "") {
            log(new Error("RealTimeAPI.QueryBuilder.validateQueryParams: 'subtrgt' is missing"));
            return false;
        }
        if (queryTargetParams[TARGET_PARAMS.ID] === "") {
            log(new Error("RealTimeAPI.QueryBuilder.validateQueryParams: 'id' is missing"));
            return false;
        }

        return true;
    };

    this.init();
};

function TargetId() {
    const UUID = "uuid";
    const STATS_SESS = "stats_sess";
    const STATS_UID = "stats_uid";
    const WITNESS_ID = "witness_id";

    this.targetId;

    this.init = function() {
        this.targetId = null;
    };

    this.get = function() {
        if (this.targetId) { 
            return this.targetId;
        }

        this.targetId = this.getAvailableId();
        if (this.targetId) { 
            return this.targetId;
        }

        this.setUUID();
        this.targetId = this.getAvailableId();

        return this.targetId;
    };

    // finds out which of the 3 possible ids is available by an order of priority
    // that id should maintain throughout the life of the add-on
    this.getAvailableId = function() {
        try {
            if (this.isIdExist(STATS_UID)) {
                var statsUID = this.getIdByName(STATS_UID);

                if (statsUID) {
                    return statsUID;
                }
            }
        } catch(ex) {
            log(ex);
        }

        if (this.isIdExist(UUID)) {
            return this.getIdByName(UUID);
        }

        if (this.isIdExist(WITNESS_ID)) {
            return this.getIdByName(WITNESS_ID);
        }

        return null;
    };

    this.isIdExist = function(idName) {
        var id = wot_prefs.getChar(idName, null);

        return id ? true : false;
    };

    this.getIdByName = function(idName) { 
        if (!this.isIdExist(idName)) { return null; }

        return wot_prefs.getChar(idName);
    };

    this.generateUUIDV4 = function() {
        var recentWin = Utils.getMostRecentWindow();

        var ts = Date.now();
        if (recentWin.performance && recentWin.performance.now) {
            ts += recentWin.performance.now();
        }

        var str = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx';
        var uuid = str.replace(/[xy]/g, function(c) {
            var r = (ts + Math.random() * 16) % 16 | 0;
            ts = Math.floor(ts / 16);

            return (c === 'x' ? r : (r&0x3|0x8)).toString(16);
        });

        return uuid;
    };

    this.setUUID = function() {
        var uuid = this.generateUUIDV4();
        wot_prefs.setChar(UUID, uuid);
    };

    this.init();
};

} catch(ex) { log(ex); }